// This webapp delegates authentication and TOTP master secret provisioning
// to the TOTP web service.

// npm install: express,body-parser,synchronize,crypto,cookie-session,request
// mv otpc.js node_modules/
// configure the web service
// set the key here
// map that response url to a handler here
// node example.js
// http://localhost:3000/

// the app is designed to run on just one server.
// the library maintains state during communication with the TOTP web service.
// so if your app is clustered,
//   create http sessions earlier than shown.
//   configure your load balancer to make them sticky.

var express = require('express');
var app = express();

var body_parser = require("body-parser");
app.use(body_parser.urlencoded({extended: true}));

//// http sessions ////

function bytes_to_string(bs) {
  var i;
  res="";
  for(i = 0; i < bs.length; ++i) res += String.fromCharCode(bs[i]);
  return res;
}
var crypto = require("crypto");
var cookie_secret = bytes_to_string(crypto.randomBytes(32));
var cookie_session = require("cookie-session");
app.use(cookie_session({"secret": cookie_secret,
                        //"secureProxy": true //for when non-node SSL
                        }));

//// end http sessions ////

//// set up the otpc library ////

var otpc = require("otpc");
otpc.init_middle(app); //breaks before body-parser
otpc.set_key("LbfZCdmo5uYZYLCF2Q6KY9YvNA-YdDZbx7LOgN1hGXY.");
app.get("/otp_response", otpc.otp_response);

//// end library setup ////

//// the sample app's web flow ////

app.get('/', function(req, res){
  res.set("Content-Type", "text/html");
  res.send("<form action=\"login\" method=\"post\">"+
           "login: <input name=\"login\"/>"+
           "<input type=\"submit\"/>"+
           "</form>");
});
app.post("/login", function (req, res) {
  res.redirect(otpc.auth(req.body.login,
                         function(session, userid, result) {
                           if(result) session["authed"]=userid;
                           else delete session["authed"];
                           return "loggedin";
                         }));
});
function assert_logged_in(u) { if (!u) throw Error("not logged in"); }
app.get("/loggedin", function(req, res) {
  var u = req.session["authed"];
  assert_logged_in(u);
  res.set("Content-Type", "text/html");
  res.send("you are logged in as " + u + "<br/>" +
           "<a href=\"changetotp\">change totp</a><br/>" +
           "<a href=\"logout\">logout</a>");
});
app.get("/changetotp", function(req,res){
  var u = req.session["authed"];
  assert_logged_in(u);
  res.redirect(otpc.prov(u, function(session,userid,nil_res){
                              return "loggedin";}));
});
app.get("/logout", function(req,res){
  delete req.session["authed"]; //or even req.session = null
  res.redirect("/");
});

//// end web flow ////

app.listen(3000);
